"""Test for multi naming style."""


def UPPER():  # [invalid-name]
    """UPPER func that will generate invalid-name error."""


def lower():
    """lower func that satisfies the prevalent group `lower`."""


def anotherlower():
    """Another lower func that satisfies the prevalent group `lower`."""
