"""
This file is used for bad-option-value's test. We needed a module that isn’t restricted by line numbers
as the tests use various line numbers to test the behaviour.

Using an empty module creates issue as you can’t disable something on line 10 if it doesn’t exist. Thus, we created an extra long file so we never run into an issue with that.
"""





























































































print(1)
